@@ -34,8 +34,9 @@ proc ::tk_dialog {w title text bitmap default args} {
     # Check that $default was properly given
     if {[string is integer -strict $default]} {
 	if {$default >= [llength $args]} {
-	    return -code error "default button index greater than number of\
-		    buttons specified for tk_dialog"
+	    return -code error -errorcode {TK DIALOG BAD_DEFAULT} \
+		"default button index greater than number of buttons\
+		specified for tk_dialog"
 	}
     } elseif {"" eq $default} {
 	set default -1
@@ -136,7 +137,7 @@ proc ::tk_dialog {w title text bitmap default args} {
 	bind $w <Return> [list $w.button$default invoke]
     }
     bind $w <<PrevWindow>> [list bind $w <Return> {[tk_focusPrev %W] invoke}]
-    bind $w <Tab> [list bind $w <Return> {[tk_focusNext %W] invoke}]
+    bind $w <<NextWindow>> [list bind $w <Return> {[tk_focusNext %W] invoke}]
 
     # 5. Create a <Destroy> binding for the window that sets the
     # button variable to -1;  this is needed in case something happens
