@@ -106,10 +106,6 @@ bind Menubutton <space> {
     tk::MbPost %W
     tk::MenuFirstEntry [%W cget -menu]
 }
-bind Menubutton <<Invoke>> {
-    tk::MbPost %W
-    tk::MenuFirstEntry [%W cget -menu]
-}
 
 # Must set focus when mouse enters a menu, in order to allow
 # mixed-mode processing using both the mouse and the keyboard.
@@ -147,25 +143,22 @@ bind Menu <ButtonRelease> {
 bind Menu <space> {
     tk::MenuInvoke %W 0
 }
-bind Menu <<Invoke>> {
-    tk::MenuInvoke %W 0
-}
 bind Menu <Return> {
     tk::MenuInvoke %W 0
 }
 bind Menu <Escape> {
     tk::MenuEscape %W
 }
-bind Menu <Left> {
+bind Menu <<PrevChar>> {
     tk::MenuLeftArrow %W
 }
-bind Menu <Right> {
+bind Menu <<NextChar>> {
     tk::MenuRightArrow %W
 }
-bind Menu <Up> {
+bind Menu <<PrevLine>> {
     tk::MenuUpArrow %W
 }
-bind Menu <Down> {
+bind Menu <<NextLine>> {
     tk::MenuDownArrow %W
 }
 bind Menu <KeyPress> {
@@ -260,7 +253,8 @@ proc ::tk::MbPost {w {x {}} {y {}}} {
     set tearoff [expr {[tk windowingsystem] eq "x11" \
 	    || [$menu cget -type] eq "tearoff"}]
     if {[string first $w $menu] != 0} {
-	error "can't post $menu:  it isn't a descendant of $w (this is a new requirement in Tk versions 3.0 and later)"
+	return -code error -errorcode {TK MENUBUTTON POST_NONCHILD} \
+	    "can't post $menu: it isn't a descendant of $w"
     }
     set cur $Priv(postedMb)
     if {$cur ne ""} {
@@ -312,9 +306,6 @@ proc ::tk::MbPost {w {x {}} {y {}}} {
 		set x [expr {[winfo rootx $w] - [winfo reqwidth $menu]}]
 		set y [expr {(2 * [winfo rooty $w] + [winfo height $w]) / 2}]
 		set entry [MenuFindName $menu [$w cget -text]]
-		if {$entry eq ""} {
-                    set entry 0
-		}
 		if {[$w cget -indicatoron]} {
 		    if {$entry == [$menu index last]} {
 			incr y [expr {-([$menu yposition $entry] \
@@ -330,14 +321,11 @@ proc ::tk::MbPost {w {x {}} {y {}}} {
 		    $menu activate $entry
 		    GenerateMenuSelect $menu
 		}
-	}
+	    }
 	    right {
 		set x [expr {[winfo rootx $w] + [winfo width $w]}]
 		set y [expr {(2 * [winfo rooty $w] + [winfo height $w]) / 2}]
 		set entry [MenuFindName $menu [$w cget -text]]
-		if {$entry eq ""} {
-                    set entry 0
-		}
 		if {[$w cget -indicatoron]} {
 		    if {$entry == [$menu index last]} {
 			incr y [expr {-([$menu yposition $entry] \
@@ -366,14 +354,12 @@ proc ::tk::MbPost {w {x {}} {y {}}} {
 		}
 	    }
 	}
-    } msg]} {
+    } msg opt]} {
 	# Error posting menu (e.g. bogus -postcommand). Unpost it and
 	# reflect the error.
 
-	set savedInfo $errorInfo
 	MenuUnpost {}
-	error $msg $savedInfo
-
+	return -options $opt $msg
     }
 
     set Priv(tearoff) $tearoff
@@ -1037,7 +1023,7 @@ proc ::tk::MenuFind {w char} {
 
 proc ::tk::TraverseToMenu {w char} {
     variable ::tk::Priv
-    if {![winfo exists $w] || $char eq ""} {
+    if {$char eq ""} {
 	return
     }
     while {[winfo class $w] eq "Menu"} {
@@ -1350,7 +1336,6 @@ proc ::tk_popup {menu x y {entry {}}} {
         tk::SaveGrabInfo $menu
 	grab -global $menu
 	set Priv(popup) $menu
-	set Priv(window) $menu
 	set Priv(menuActivated) 1
 	tk_menuSetFocus $menu
     }
